<?php

/**
 * The admin-specific functionality of the plugin.
 *
 * @link       https://welaunch.io/plugins/wordpress-multilingual-multisite/
 * @since      1.0.0
 *
 * @package    WordPress_Multilingual_Multisite
 * @subpackage WordPress_Multilingual_Multisite/admin
 */

/**
 * The admin-specific functionality of the plugin.
 *
 * Defines the plugin name, version, and two examples hooks for how to
 * enqueue the admin-specific stylesheet and JavaScript.
 *
 * @package    WordPress_Multilingual_Multisite
 * @subpackage WordPress_Multilingual_Multisite/admin
 * @author     Daniel Barenkamp <support@welaunch.io>
 */
class WordPress_Multilingual_Multisite_Network_Query extends WordPress_Multilingual_Multisite {

	/**
	 * The ID of this plugin.
	 *
	 * @since    1.0.0
	 * @access   protected
	 * @var      string    $plugin_name    The ID of this plugin.
	 */
	protected $plugin_name;

	/**
	 * The version of this plugin.
	 *
	 * @since    1.0.0
	 * @access   protected
	 * @var      string    $version    The current version of this plugin.
	 */
	protected $version;

	/**
	 * Initialize the class and set its properties.
	 *
	 * @since    1.0.0
	 * @param      string    $plugin_name       The name of this plugin.
	 * @param      string    $version    The version of this plugin.
	 */
	public function __construct( $plugin_name, $version ) {

		$this->plugin_name = $plugin_name;
		$this->version = $version;

	}

	public function init() {
		global $wordpress_multilingual_multisite;

		$this->options = $wordpress_multilingual_multisite;

		if (!$this->get_option('enable'))
		{
			return FALSE;
		}
	}

    public function get_posts($args)
    {
        global $blogs_data;

        $sites = $blogs_data;
        $posts = array();
            
        $args['post_status'] = array('publish', 'pending', 'draft', 'future', 'private', 'trash');
        if(isset($args['sites_not']) && !empty($args['sites_not'])) {
            if(is_array($args['sites_not'])) {
                foreach ($args['sites_not'] as $site_not) {
                    unset($sites[$site_not]);
                }
            } elseif(is_int($args['sites_not'])) {
                unset($sites[$args['sites_not']]);
            }
        }

        if(isset($args['sites_in']) && !empty($args['sites_in'])) {
            if(!is_array($args['sites_in'])) {
                $args['sites_in'] = array($args['sites_in']);
            }
            $sites = array_intersect_key(array_flip( $args['sites_in'] ), $sites);
        }

        foreach ($sites as $site_id => $site) {

            switch_to_blog($site_id);

            $tmp = get_posts($args);

            $posts[$site_id] = $tmp;

            restore_current_blog();
        }

        return $posts;
    }

    public function get_terms($args)
    {
        global $blogs_data;

        $sites = $blogs_data;
        $terms = array();
        
        if(isset($args['sites_not']) && !empty($args['sites_not'])) {
            if(is_array($args['sites_not'])) {
                foreach ($args['sites_not'] as $site_not) {
                    unset($sites[$site_not]);
                }
            } elseif(is_int($args['sites_not'])) {
                unset($sites[$args['sites_not']]);
            }
        }

        if(isset($args['sites_in']) && !empty($args['sites_in'])) {
            if(!is_array($args['sites_in'])) {
                $args['sites_in'] = array($args['sites_in']);
            }
            $sites = array_intersect_key(array_flip( $args['sites_in'] ), $sites);
        }

        foreach ($sites as $site_id => $site) {

            switch_to_blog($site_id);

            $tmp = get_terms($args);

            $terms[$site_id] = $tmp;

            restore_current_blog();
        }

        return $terms;
    }

    public function get_wc_attribute_taxonomies()
    {

        global $blogs_data;

        $sites = $blogs_data;
        $terms = array();
        
        if(isset($args['sites_not']) && !empty($args['sites_not'])) {
            if(is_array($args['sites_not'])) {
                foreach ($args['sites_not'] as $site_not) {
                    unset($sites[$site_not]);
                }
            } elseif(is_int($args['sites_not'])) {
                unset($sites[$args['sites_not']]);
            }
        }

        if(isset($args['sites_in']) && !empty($args['sites_in'])) {
            if(!is_array($args['sites_in'])) {
                $args['sites_in'] = array($args['sites_in']);
            }
            $sites = array_intersect_key(array_flip( $args['sites_in'] ), $sites);
        }

        foreach ($sites as $site_id => $site) {

            switch_to_blog($site_id);

            $tmp = wc_get_attribute_taxonomies();

            $terms[$site_id] = $tmp;

            restore_current_blog();
        }

        return $terms;
    }

    public function get_wc_attribute_terms($args)
    {

        global $blogs_data;

        $sites = $blogs_data;
        $terms = array();
        
        if(isset($args['sites_not']) && !empty($args['sites_not'])) {
            if(is_array($args['sites_not'])) {
                foreach ($args['sites_not'] as $site_not) {
                    unset($sites[$site_not]);
                }
            } elseif(is_int($args['sites_not'])) {
                unset($sites[$args['sites_not']]);
            }
        }

        if(isset($args['sites_in']) && !empty($args['sites_in'])) {
            if(!is_array($args['sites_in'])) {
                $args['sites_in'] = array($args['sites_in']);
            }
            $sites = array_intersect_key(array_flip( $args['sites_in'] ), $sites);
        }

        foreach ($sites as $site_id => $site) {
            switch_to_blog($site_id);

            $terms_by_site[$site_id] = array();

            $attribute_taxonomies = wc_get_attribute_taxonomies();
            if(empty($attribute_taxonomies)) {
                restore_current_blog();
                continue;
            }

            $tmp = array();
            foreach ($attribute_taxonomies as $attribute_taxonomy) {

                $all_terms = get_terms(array('hide_empty' => false));
                $terms = array();
                foreach ($all_terms as $all_term) {
                    if($all_term->taxonomy == 'pa_' . $attribute_taxonomy->attribute_name) {
                        $terms[] = $all_term;
                    }
                }

                $attribute_taxonomy->terms = $terms;
                $terms_by_site[$site_id][] = $attribute_taxonomy;
            }

            restore_current_blog();
        }
        return $terms_by_site;
    }
}