<?php

/**
 * The admin-specific functionality of the plugin.
 *
 * @link       https://welaunch.io/plugins/wordpress-multilingual-multisite/
 * @since      1.0.0
 *
 * @package    WordPress_Multilingual_Multisite
 * @subpackage WordPress_Multilingual_Multisite/admin
 */

/**
 * The admin-specific functionality of the plugin.
 *
 * Defines the plugin name, version, and two examples hooks for how to
 * enqueue the admin-specific stylesheet and JavaScript.
 *
 * @package    WordPress_Multilingual_Multisite
 * @subpackage WordPress_Multilingual_Multisite/admin
 * @author     Daniel Barenkamp <support@welaunch.io>
 */
class WordPress_Multilingual_Multisite_Post_Copier extends WordPress_Multilingual_Multisite {

    protected $plugin_name;
    protected $version;

    /**
     * Initialize the class and set its properties.
     *
     * @since    1.0.0
     * @param      string    $plugin_name       The name of this plugin.
     * @param      string    $version    The version of this plugin.
     */
    public function __construct( $plugin_name, $version, $translation_tables, $terms_copier, $current_blog_id ) 
    {

        $this->plugin_name = $plugin_name;
        $this->version = $version;

        $this->translation_tables = $translation_tables;
        $this->terms_copier = $terms_copier;

        $this->current_blog_id = $current_blog_id;
        $this->prefix = 'wordpress_multilingual_multisite_';
    }

    public function init() 
    {
        global $wordpress_multilingual_multisite;

        $this->options = $wordpress_multilingual_multisite;

        if (!$this->get_option('enable')) {
            return FALSE;
        }

        if (!$this->get_option('postCopierEnable')) {
            return FALSE;
        }

        $post_types = get_post_types();
        foreach ($post_types as $post_type) {
            add_filter( 'bulk_actions-edit-' . $post_type, array($this, 'add_bulk_action'), 20, 1 );
            add_filter( 'handle_bulk_actions-edit-' . $post_type, array($this, 'handle_bulk_action'), 10, 3 );
        }
    }

    public function copy_single()
    {
        if(!isset($_GET['wp-trans-copy']) || empty($_GET['wp-trans-copy']) || $_GET['wp-trans-copy'] !== "post") {
            return false;
        }

        if( !isset($_GET['source_blog']) || empty($_GET['source_blog']) ||
            !isset($_GET['source_id']) || empty($_GET['source_id'])
        ) {
            return false;
        }

        $source_blog = $_GET['source_blog'];
        $source_id = $_GET['source_id'];

        $new_post_id = $this->copy_post($source_blog, $source_id);
        $original_post_type = get_post_type($new_post_id);

        // Regenerate WC lookup table
        if(function_exists('wc_update_product_lookup_tables')) {
            wc_update_product_lookup_tables();
        }

        wp_redirect( admin_url( 'post.php?action=edit&post=' . $new_post_id . '&post_type=' . $original_post_type . '&source_blog=' . $source_blog . '&source_id=' . $source_id) );

    }

    public function copy_multiple()
    {
        if(!isset($_GET['wp-trans-copy']) || empty($_GET['wp-trans-copy']) || $_GET['wp-trans-copy'] !== "post") {
            return false;
        }

        if( !isset($_GET['source_blog']) || empty($_GET['source_blog']) ||
            !isset($_GET['source_ids']) || empty($_GET['source_ids'])
        ) {
            return false;
        }

        $source_blog = $_GET['source_blog'];
        $source_ids = explode(',', $_GET['source_ids']);

        foreach ($source_ids as $source_id) {
            $this->copy_post($source_blog, $source_id);
        }

        // Regenerate WC lookup table
        if(function_exists('wc_update_product_lookup_tables')) {
            wc_update_product_lookup_tables();
        }

        wp_redirect( admin_url( 'edit.php?post_type=' . $_GET['post_type']) );
    }

    public function copy_post($source_blog, $source_id, $redirect = false, $publish = false, $returnID = false, $parent = 0)
    {
        global $wpdb;
        
        $original_source_blog = $source_blog;
        switch_to_blog($source_blog);

        $original_post = (array) get_post($source_id);
        $original_post_meta = get_post_meta($source_id);
        if(isset($_POST) && !empty($_POST)) {
            $original_post_meta = array_merge($_POST, $original_post_meta);
            $original_post_meta = array_diff_key($original_post_meta, $original_post);
        }

        $children = false; // get_children( 'post_parent=' . $source_id . '&post_type=product_variation' );
        $childrenQueryArgs = array(
            'post_type'      => get_post_types(),
            'posts_per_page' => -1,
            'post_parent'    => $source_id,
        );
        $childrenQuery = new WP_Query( $childrenQueryArgs );
        if(isset($childrenQuery->posts) && !empty($childrenQuery->posts)) {
            $children = $childrenQuery->posts;
        }
        
        $original_post_type = get_post_type($source_id);       

        $original_taxonomy_terms = array();
        $original_taxonomies = get_object_taxonomies( $original_post ); //get_taxonomies();

        if($publish) {
            $original_post['post_status'] = 'publish';
        }

        // add and register missing woo attributes
        if(class_exists( 'woocommerce' ) ) {

            $source_attribute_taxonomy = $wpdb->get_results( "SELECT * FROM {$wpdb->prefix}woocommerce_attribute_taxonomies" );

            $source_attribute_taxonomies = array();
            if(!empty($source_attribute_taxonomy)) {

                $not_existing_attributes = array();
                foreach ($source_attribute_taxonomy as $_source_attribute_taxonomy) {
                    $source_attribute_taxonomy_name = 'pa_' . $_source_attribute_taxonomy->attribute_name;
                    if(!in_array($source_attribute_taxonomy_name, $original_taxonomies)) {

                        $not_existing_attributes[] = (array) $_source_attribute_taxonomy;
                        register_taxonomy($source_attribute_taxonomy_name, 'product');
                        $original_taxonomies[] = $source_attribute_taxonomy_name;
                    }
                }                
            }
        }

        // uncategegorized
        $default_source_terms = array(
            get_option('default_category'),
            get_option('default_product_cat'),
        );         

        foreach ($original_taxonomies as $original_taxonomy) {
            $original_taxonomy_terms[$original_taxonomy] = wp_get_post_terms($source_id, $original_taxonomy,  array("fields" => "all"));
        }

        // get original images
        $sourceImageIds = array();
        $sourceImages = array();
        if($this->get_option('postCopierCopyImages')) {

            preg_match_all('/id="(.+?)"/', $original_post['post_content'], $matches);
            if (isset($matches[0]) && !empty($matches[0]) & isset($matches[1]) && !empty($matches[1]))
            {
                $sourceImageIds = array_merge($sourceImageIds, $matches[1] );
            }


            preg_match_all('/img="(.+?)"/', $original_post['post_content'], $matches);
            if (isset($matches[0]) && !empty($matches[0]) & isset($matches[1]) && !empty($matches[1]))
            {
                $sourceImageIds = array_merge($sourceImageIds, $matches[1] );
            }

            preg_match_all('/image="(.+?)"/', $original_post['post_content'], $matches);
            if (isset($matches[0]) && !empty($matches[0]) & isset($matches[1]) && !empty($matches[1]))
            {
                $sourceImageIds = array_merge($sourceImageIds, $matches[1] );
            }

            preg_match_all('/image_ids="(.+?)"/', $original_post['post_content'], $matches);
            if (isset($matches[0]) && !empty($matches[0]) & isset($matches[1]) && !empty($matches[1]))
            {
                $sourceImageIds = array_merge($sourceImageIds, $matches[1] );
            }

            preg_match_all('/ids="(.+?)"/', $original_post['post_content'], $matches);
            if (isset($matches[0]) && !empty($matches[0]) & isset($matches[1]) && !empty($matches[1]))
            {
                $sourceImageIds = array_merge($sourceImageIds, $matches[1] );            
            }

            $tmp = array();
            foreach($sourceImageIds as $sourceImageKey => $sourceImageValue) {
                $tmp = array_merge($tmp, explode(',', $sourceImageValue));
            }
            $sourceImageIds = array_unique( array_filter( $tmp ) );

            foreach($sourceImageIds as $sourceImageIdKey => $sourceImageId) {

                $imageToUpload = get_post($sourceImageId);
                if (!$imageToUpload || !isset($imageToUpload->ID) || $imageToUpload->post_type != "attachment") {
                    unset($sourceImageIds[$sourceImageIdKey]);
                    continue;
                }
                $sourceImages[] = $imageToUpload;
            }

        }

        restore_current_blog();

        if ( $parent > 0 ) {
            $original_post['post_parent'] = $parent;
        }

        unset($original_post['ID']);

        $excludeDataKeys = array();
        if(!empty($this->get_option('postCopierExcludeDataKeys'))) {
            foreach ($excludeDataKeys as $excludeDataKey) {
                unset($original_post[$excludeDataKey]);
            }
        }

        remove_all_filters("content_save_pre");
        $new_post_id = wp_insert_post($original_post);

        // Update Metas
        $excludeMetaKeys = array();
        if(!empty($this->get_option('postCopierExcludeMetaKeys'))) {
            $excludeMetaKeys = explode(',', $this->get_option('postCopierExcludeMetaKeys'));
        }

        foreach($original_post_meta as $original_post_meta_key => $original_post_meta_value) {

            if(in_array($original_post_meta_key, $excludeMetaKeys)) {
                continue;
            }

            if(is_array($original_post_meta_value)) {
                $original_post_meta_value = $original_post_meta_value[0];
            }

            if(is_serialized($original_post_meta_value)) {
                $original_post_meta_value = unserialize($original_post_meta_value);
            }

            // Avoid update-post_meta wp_unsplash & serialize stuff (fucked up)
            if($original_post_meta_key == "_elementor_data") {
                $table = $wpdb->prefix . 'postmeta';
                $wpdb->query( $wpdb->prepare("INSERT INTO $table (meta_key, meta_value, post_id) VALUES('_elementor_data', %s, %d)", $original_post_meta_value, $new_post_id ) );
            } else {
                // if key ==  pa_ => get translated taxonomy
                update_post_meta($new_post_id, $original_post_meta_key, $original_post_meta_value);
            }
        }

        if($this->get_option('postCopierCopyTerms')) {

            // Update Terms
            $this->translation_tables->set_table('terms_translations');


            if(class_exists( 'woocommerce' ) && !empty($not_existing_attributes)) {
                foreach ($not_existing_attributes as $not_existing_attribute) {
                    $wpdb->insert( $wpdb->prefix . 'woocommerce_attribute_taxonomies', $not_existing_attribute );

                    do_action( 'woocommerce_attribute_added', $wpdb->insert_id, $not_existing_attribute );

                    flush_rewrite_rules();
                    delete_transient( 'wc_attribute_taxonomies' );
                }
            }

            foreach ($original_taxonomy_terms as $taxonomy => $terms) {
                
                if(empty($terms)) {
                    continue;
                }

                if ( ! taxonomy_exists( $taxonomy ) ) {
                    register_taxonomy( $taxonomy, $original_post_type );
                }

                foreach ($terms as $term) {
 
                    if(in_array($term->term_id, $default_source_terms)) {
                        continue;
                    }

                    // Connect Term exists
                    $translated_terms = $this->translation_tables->get_translated_items_by_item_id($source_blog, $term->term_id);
                    if(isset($translated_terms[$this->current_blog_id]) && !empty($translated_terms[$this->current_blog_id])) {
                        $term_found = array( (int) $translated_terms[$this->current_blog_id]);
                        wp_set_object_terms( $new_post_id, $term_found, $taxonomy, true);
                        continue;
                    }

                    // Connected Term not exists -> check by name
                    if(term_exists( $term->name, $taxonomy)) {
                        wp_set_object_terms( $new_post_id, $term->name, $taxonomy, true);
                    // if not exists insert? 
                    } else {
                        // use term copier to keep translations ...
                        // $new_term =  wp_insert_term($term->name, $taxonomy);
                        $new_term = $this->terms_copier->copy($source_blog, $term->term_id, $this->current_blog_id, false);  // wp_insert_term($term->name, $taxonomy);
                        if(isset($new_term['term_id'])) {
                            $term_found = array( (int) $new_term['term_id']);
                            wp_set_object_terms( $new_post_id, $term_found, $taxonomy, true);
                        }
                    }
                }
            }
        }
        

        if($this->get_option('postCopierCopyImages')) {

            // Post attached images
            $this->duplicate_media($source_id, $source_blog, $new_post_id, $this->current_blog_id);

            // Images inside page builders etc.
            $newImageIds = array();
            if (!empty($sourceImages))
            {
                foreach ($sourceImages as $sourceImage)
                {
                    $upload = $this->fetch_remote_file($sourceImage->guid);
                    // exists
                    if (is_int($upload))
                    {
                        $newImageIds[] = $upload;
                    }
                    // New Image
                    else
                    {
                        if (is_wp_error($upload))
                        {
                            var_dump($upload);
                            die('Image Upload error: ' . $upload->get_error_message());
                        }

                        $newImage = (array)$sourceImage;
                        unset($newImage['ID']);
                        unset($newImage['guid']);

                        if ($info = wp_check_filetype($upload['file']))
                        {
                            $newImage['post_mime_type'] = $info['type'];
                        }
                        else
                        {
                            $newImageIds[] = "";
                            continue;
                        }

                        // change to NEW post ID if?
                        $newImage['post_parent'] = $new_post_id;

                        $newImage['guid'] = $upload['url'];

                        require_once (ABSPATH . 'wp-admin/includes/image.php');

                        $newImageId = wp_insert_attachment($newImage, $upload['file']);
                        wp_update_attachment_metadata($newImageId, wp_generate_attachment_metadata($newImageId, $upload['file']));

                        if ($newImageId)
                        {
                            $newImageIds[] = $newImageId;
                        }
                        else
                        {
                            $newImageIds[] = "";
                        }
                    }
                }

                if(count($sourceImageIds) == count($newImageIds)) {
                    $new_post = get_post($new_post_id);
                    $new_post->post_content = str_replace($sourceImageIds, $newImageIds, $new_post->post_content);
                    wp_update_post($new_post);
                }
            }

            // str_replace($sourceImageIds, $newImageIds, $post_content);
        }

        // Translation Update
        $this->translation_tables->set_table('posts_translations');
        $translated_posts = $this->translation_tables->get_translated_items_by_item_id($source_blog, $source_id);
        if(!$translated_posts) {
            $translated_posts = array(
                $source_blog => $source_id
            );
        }
        $translated_posts[$this->current_blog_id] = $new_post_id;
        $translated_posts['type'] = $original_post_type;

        unset($translated_posts['id']);
        unset($translated_posts['data']);

        $exists = $this->translation_tables->get_translated_items_by_translated_items($translated_posts);
        if($exists) {
            $this->translation_tables->update_translated_items($exists['id'], $this->current_blog_id, $new_post_id, $translated_posts);
        } else {
            $this->translation_tables->insert_translated_items($this->current_blog_id, $new_post_id, $translated_posts);
        }

        if ( $children ) {
            foreach ( $children as $child ) {
                $this->copy_post( $original_source_blog, $child->ID, false, $child->post_status, false, $new_post_id );
            }
        }

        if($redirect) {
            wp_redirect( admin_url( 'post.php?action=edit&post=' . $new_post_id . '&post_type=' . $original_post_type . '&source_blog=' . $source_blog . '&source_id=' . $source_id) );
        } else {
            return $new_post_id;
        }
    }

    /**
     * duplicate media
     *
     * @param  WP_Post $post Post object
     * @return array
     */
    public function duplicate_media( $original_post_id, $source_blog, $new_post_id, $target_blog) {

        switch_to_blog($source_blog);

        // Get all media
        $images = $this->get_all_images( $original_post_id );

        restore_current_blog();

        if(empty($images)){
            return FALSE;
        }

        $new_gallery_images = array();
        $new_variation_gallery_images = array();
        foreach ( $images as $key => $image ) {

            if(!$image || !isset($image->ID)) {
                continue;
            }

            switch_to_blog($source_blog);
            $image_alt_text = get_post_meta( $image->ID, '_wp_attachment_image_alt', true );
            $metadata = wp_get_attachment_metadata( $image->ID );

            // Is the image a thumbnail?
            $is_thumbnail = false;
            if ( ! empty( $image->is_thumbnail ) )
                $is_thumbnail = true;

            $is_product_variation_gallery_image = false;
            if ( ! empty( $image->is_product_variation_gallery_image ) ) {
                $is_product_variation_gallery_image = true;
            }
            restore_current_blog();

            switch_to_blog( $target_blog );

            $new_image = (array)$image;
            unset( $new_image['ID'] );
            unset( $new_image['guid'] );

            $upload = $this->fetch_remote_file( $image->guid );
            
            if(is_int($upload)){
                $new_image_id = $upload;
            } else {
                if ( is_wp_error( $upload ) ) {
                    if($this->get_option('imageDebug')) {
                        die('Image Upload error: ' . $upload->get_error_message());
                    }
                    continue;
                }

                if ( $info = wp_check_filetype( $upload['file'] ) )
                    $new_image['post_mime_type'] = $info['type'];
                else
                    continue;

                $new_image['post_parent'] = $new_post_id;

                $new_image['guid'] = $upload['url'];

                require_once( ABSPATH . 'wp-admin/includes/image.php' );

                $new_image_id = wp_insert_attachment( $new_image, $upload['file'] );
                wp_update_attachment_metadata( $new_image_id, wp_generate_attachment_metadata( $new_image_id, $upload['file'] ) );

                // alt text
                update_post_meta( $new_image_id, '_wp_attachment_image_alt', $image_alt_text );
            }

            if ( $is_thumbnail ){
                set_post_thumbnail( $new_post_id, $new_image_id );
            } 
            elseif($is_product_variation_gallery_image) {
                $new_variation_gallery_images[] = $new_image_id;
            }
            else {
                $new_gallery_images[] = $new_image_id;
            }
        }

        $new_gallery_images = implode(',', $new_gallery_images);
        if(!empty($new_gallery_images)) {
            update_post_meta( $new_post_id, '_product_image_gallery', $new_gallery_images);
        }

        $new_variation_gallery_images = implode(',', $new_variation_gallery_images);
        if(!empty($new_variation_gallery_images)) {
            update_post_meta( $new_post_id, 'variation_image_gallery', $new_variation_gallery_images);
        }

        restore_current_blog();
    }

    /**
     * Get images
     *
     * @param  int $item_id
     * @return array
     */
    public function get_all_images( $item_id ) {

        $orig_post = get_post( $item_id );
        $all_images = array();

        // get Thumbnail
        $thumbnail = get_post( get_post_thumbnail_id( $orig_post->ID ) );

        // get product gallery images
        $product_gallery_images = get_post_meta($orig_post->ID, '_product_image_gallery');
        if(!empty($product_gallery_images)){
            $product_gallery_images = explode(',', $product_gallery_images[0]);

            $args = array(
                'post_type' => 'attachment',
                'posts_per_page' => -1,
                'post__in' => $product_gallery_images
            );
            $product_gallery_images = get_posts($args);
        }

        // get product variation gallery images
        $product_variation_gallery_images = get_post_meta($orig_post->ID, 'variation_image_gallery');
        if(!empty($product_variation_gallery_images)){
            $product_variation_gallery_images = explode(',', $product_variation_gallery_images[0]);

            $args = array(
                'post_type' => 'attachment',
                'posts_per_page' => -1,
                'post__in' => $product_variation_gallery_images
            );
            $product_variation_gallery_images = get_posts($args);
            $temp = array();
            foreach ($product_variation_gallery_images as $product_variation_gallery_image) {
                $asd = $product_variation_gallery_image;
                $asd->is_product_variation_gallery_image = true;
                $temp[] = $asd;
            }
            $product_variation_gallery_images = $temp;
        }

        if ( ! empty( $thumbnail ) ) {
            $thumbnail->is_thumbnail = true;
        }

        $all_images = array_merge( array( $thumbnail ), $product_gallery_images, $product_variation_gallery_images );

        if(empty($all_images)){
            return FALSE;
        }

        // Removing duplicate attachments
        $all_images_no_duplicates = array();
        $temp = array();
        foreach ( $all_images as $attachment ) {
            if ( ! in_array( $attachment->ID, $temp ) ) {
                $temp[] = $attachment->ID;
                $all_images_no_duplicates[] = $attachment;
            }
        }

        $all_images = $all_images_no_duplicates;

        return $all_images;
    }

    /** 
     * Fetch an image and download it.
     *
     * @return WP_Error/Array Image properties/Error
     */
    public function fetch_remote_file( $url )
    {
        $file = basename( $url );
        $file_parts = pathinfo($file);
        $file_name = $file_parts['filename'];
        $file_extension = $file_parts['extension'];

        $args = array(
            'name' => $file_name,
            'post_type' => 'attachment',
            'posts_per_page' => -1,
        );

        $check_file_exists = get_posts($args);


        if(!empty($check_file_exists)){
            $fileExists = $check_file_exists[0]->ID;
            return $fileExists;
        }

        $upload = wp_upload_bits( $file, null, 0 );

        if ( $upload['error'] )
            return new WP_Error( 'upload_dir_error', $upload['error'] );

        $fetch_settings = array(
            'stream' => true,
            'filename' => $upload['file'],
            'sslverify' => false
        );


        if($this->get_option('advancedBasicAuth')) {
            $fetch_settings['headers'] = array(
                'Authorization' => 'Basic ' . base64_encode( $this->get_option('advancedBasicAuthUser') . ':' . $this->get_option('advancedBasicAuthPassword') )
            );
        }

        $response = wp_remote_get( $url, $fetch_settings );

        if(is_wp_error($response)) {
            return $response;
        }

        if ( ! $response ) {
            @unlink( $upload['file'] );
            return new WP_Error( 'import_file_error', sprintf( __('Remote server did not respond for file: %s', 'wordpress-multilingual-multisite' ), $url ) );
        }

        if(is_wp_error($response)) {
            return $response;
        }

        if ( $response['response']['code'] != '200' ) {
            @unlink( $upload['file'] );
            return new WP_Error( 'import_file_error', sprintf( __('Remote server returned error response %1$d %2$s', 'wordpress-multilingual-multisite' ), esc_html($response['response']['code']), get_status_header_desc($response['response']['code']) ) );
        }

        $filesize = filesize( $upload['file'] );
        // error with SVG
        // if ( isset( $response['headers']['content-length'] ) && $filesize != $response['headers']['content-length'] ) {
        //     @unlink( $upload['file'] );
        //     return new WP_Error( 'import_file_error', __('Remote file is incorrect size', 'wordpress-multilingual-multisite' ) );
        // }

        if ( 0 == $filesize ) {
            @unlink( $upload['file'] );
            return new WP_Error( 'import_file_error', __('Zero size file downloaded', 'wordpress-multilingual-multisite' ) );
        }

        $max_size = (int) apply_filters( 'mcc_attachment_size_limit', 0 );
        if ( ! empty( $max_size ) && $filesize > $max_size ) {
            @unlink( $upload['file'] );
            return new WP_Error( 'import_file_error', sprintf(__('Remote file is too large, limit is %s', 'wordpress-multilingual-multisite' ), size_format($max_size) ) );
        }

        return $upload;
    }

    public function add_bulk_action($bulk_actions)
    {
        global $blogs_data;

        unset($blogs_data[$this->current_blog_id]);
        foreach ($blogs_data as $blog_id => $blog_data) {
            $bulk_actions['copy_to_' . $blog_id] = sprintf( esc_html__( 'Copy to: %s', 'woocommerce-customizations'), $blog_data['name']);
        }

        return $bulk_actions;
    }

    public function handle_bulk_action($redirect_to, $action, $post_ids)
    {
        $checkAction = strpos($action, 'copy_to_');
        
        if ( $checkAction === false ) {
            return $redirect_to;
        }

        $actionSplitted = explode('_', $action);

        if(isset($actionSplitted[2])) {

            $blog_id = intval( $actionSplitted[2] );
            $blog_url = get_admin_url($blog_id, 'edit.php?post_type=' . $_GET['post_type']);
            $blog_url .= '&wp-trans-copy=post&source_blog=' . $this->current_blog_id . '&source_ids=' . implode(',', $post_ids);   
        }

        return $blog_url;
    }


    public function bulk_action_printer_assigned_admin_notice()
    {
        if ( ! empty( $_REQUEST['bulk_printer_assigned'] ) ) {
            $drafts_count = intval( $_REQUEST['bulk_printer_assigned'] );

            printf(
                '<div id="message" class="updated fade">' .
                _n( '%s order assigned to printers.', '%s orders assigned to printers.', $drafts_count, 'woocommerce-customizations' )
                . '</div>',
                $drafts_count
            );
        }
    }
}