<?php

/**
 * The admin-specific functionality of the plugin.
 *
 * @link       https://welaunch.io/plugins/wordpress-multilingual-multisite/
 * @since      1.0.0
 *
 * @package    WordPress_Multilingual_Multisite
 * @subpackage WordPress_Multilingual_Multisite/admin
 */

/**
 * The admin-specific functionality of the plugin.
 *
 * Defines the plugin name, version, and two examples hooks for how to
 * enqueue the admin-specific stylesheet and JavaScript.
 *
 * @package    WordPress_Multilingual_Multisite
 * @subpackage WordPress_Multilingual_Multisite/admin
 * @author     Daniel Barenkamp <support@welaunch.io>
 */
class WordPress_Multilingual_Multisite_Terms_Copier extends WordPress_Multilingual_Multisite {

	protected $plugin_name;
	protected $version;

	/**
	 * Initialize the class and set its properties.
	 *
	 * @since    1.0.0
	 * @param      string    $plugin_name       The name of this plugin.
	 * @param      string    $version    The version of this plugin.
	 */
	public function __construct( $plugin_name, $version, $translation_tables, $current_blog_id ) 
    {

		$this->plugin_name = $plugin_name;
		$this->version = $version;

        $this->translation_tables = $translation_tables;

        $this->current_blog_id = $current_blog_id;
        $this->prefix = 'wordpress_multilingual_multisite_';
	}

	public function init() 
    {
		global $wordpress_multilingual_multisite;
		$this->options = $wordpress_multilingual_multisite;
	}

    public function copy($source_blog = null, $source_id = null, $target_blog = null, $redirect = true)
    {
        if(!isset($_GET['wp-trans-copy']) || empty($_GET['wp-trans-copy']) || $_GET['wp-trans-copy'] !== "term") {
            if(!$source_blog || !$source_id || !$target_blog) {
                return false;
            }
        }

        global $wp_taxonomies;

        if(!$source_blog) {
            $source_blog = $_GET['source_blog'];
        }

        if(!$source_id) {
            $source_id = $_GET['source_id'];
        }

        if(!$target_blog) {
            $target_blog = $this->current_blog_id;
        }

        // Get Original Data
        switch_to_blog($source_blog);

        $original_term = get_term($source_id);

        if($original_term->parent) {
            $original_parent_term = get_term($original_term->parent);
        }

        $original_post_type = $wp_taxonomies[$original_term->taxonomy]->object_type[0];

        $original_term_metas = get_term_meta($source_id);

        restore_current_blog();
        
        unset($original_term->term_id);

        $excludeDataKeys = array();
        if(!empty($this->get_option('termCopierExcludeDataKeys'))) {
            $excludeDataKeys = explode(',', $this->get_option('termCopierExcludeDataKeys'));
            foreach ($excludeDataKeys as $excludeDataKey) {
                $original_term->$excludeDataKey = "";
            }
        }

        // parent term exists
        if($original_term->parent) {

            $parentTermExists = term_exists( $original_parent_term->slug, $original_term->taxonomy);
            if(!$parentTermExists) {
                $original_term->parent = $this->copy($source_blog, $original_term->parent, $target_blog, false)['term_id'];
                switch_to_blog($target_blog);
            } else {
                $original_term->parent = $parentTermExists['term_id'];
            }
        }

        $term_data = array(
            'description' => $original_term->description,
            'parent' => $original_term->parent,
            'slug' => $original_term->slug,
        );
        $new_term = wp_insert_term($original_term->name, $original_term->taxonomy, $term_data);

        // Term Exists
        if(is_wp_error($new_term)) {

            $error_string = $new_term;
            
            if(isset($error_string->error_data['term_exists']) && !empty($error_string->error_data['term_exists'])) {

                $termExists = $error_string->error_data['term_exists'];


                $translated_terms = array(
                    'type' => $original_term->name,
                    $target_blog => $termExists,
                    $source_blog => $source_id
                );

                $this->translation_tables->set_table('terms_translations');
                $exists = $this->translation_tables->get_translated_items_by_translated_items($translated_terms);

                if($exists) {
                    $this->translation_tables->update_translated_items($exists['id'], $target_blog, $term_id, $translated_terms);
                } else {
                    $this->translation_tables->insert_translated_items($target_blog, $term_id, $translated_terms);
                }

                if($redirect == false) {
                    return $error_string['error_data']['term_exists'];
                } else {
                    wp_redirect( admin_url( 'term.php?taxonomy=' . $original_term->taxonomy . '&tag_ID=' . $termExists . '&post_type=' . $original_post_type . '&source_blog=' . $source_blog . '&source_id=' . $source_id) );
                }
                
            }
            return false;
        }

        // Update Metas
        $excludeMetaKeys = array();
        if(!empty($this->get_option('termCopierExcludeMetaKeys'))) {
            $excludeMetaKeys = explode(',', $this->get_option('termCopierExcludeMetaKeys'));
        }

        if(!empty($original_term_metas)) {
            foreach ($original_term_metas as $original_term_meta_key => $original_term_meta_value) {
                if(in_array($original_term_meta_key, $excludeMetaKeys)) {
                    continue;
                }
                update_term_meta($new_term['term_id'], $original_term_meta_key, $original_term_meta_value);
            }
        }


        $translated_terms = array(
            'type' => $original_term->name,
            $target_blog => $new_term['term_id'],
            $source_blog => $source_id
        );

        // Translation Update
        $this->translation_tables->set_table('terms_translations');
        $exists = $this->translation_tables->get_translated_items_by_translated_items($translated_terms);

        if($exists) {
            $this->translation_tables->update_translated_items($exists['id'], $target_blog, $new_term['term_id'], $translated_terms);
        } else {
            $this->translation_tables->insert_translated_items($target_blog, $term_id, $translated_terms);
        }

        if($redirect) {
            wp_redirect( admin_url( 'term.php?taxonomy=' . $original_term->taxonomy . '&tag_ID=' . $new_term['term_id'] . '&post_type=' . $original_post_type . '&source_blog=' . $source_blog . '&source_id=' . $source_id) );
        }

        return $new_term;
    }
}