<?php

/**
 * The admin-specific functionality of the plugin.
 *
 * @link       https://welaunch.io/plugins/wordpress-multilingual-multisite/
 * @since      1.0.0
 *
 * @package    WordPress_Multilingual_Multisite
 * @subpackage WordPress_Multilingual_Multisite/admin
 */

/**
 * The admin-specific functionality of the plugin.
 *
 * Defines the plugin name, version, and two examples hooks for how to
 * enqueue the admin-specific stylesheet and JavaScript.
 *
 * @package    WordPress_Multilingual_Multisite
 * @subpackage WordPress_Multilingual_Multisite/admin
 * @author     Daniel Barenkamp <support@welaunch.io>
 */
class WordPress_Multilingual_Multisite_WooCommerce extends WordPress_Multilingual_Multisite {

	protected $plugin_name;
	protected $version;

	/**
	 * Initialize the class and set its properties.
	 *
	 * @since    1.0.0
	 * @param      string    $plugin_name       The name of this plugin.
	 * @param      string    $version    The version of this plugin.
	 */
	public function __construct( $plugin_name, $version, $translation_tables, $current_blog_id ) 
    {

		$this->plugin_name = $plugin_name;
		$this->version = $version;

        $this->translation_tables = $translation_tables;

        $this->current_blog_id = $current_blog_id;
        $this->prefix = 'wordpress_multilingual_multisite_';
	}

	public function init() 
    {
		global $wordpress_multilingual_multisite;

		$this->options = $wordpress_multilingual_multisite;

		if (!$this->get_option('enable')) {
			return FALSE;
		}
	}

    public function sync_product_meta( $meta_id, $object_id, $meta_key, $_meta_value )
    {
        if(!$this->get_option('woocommerceDataSync')) {
            return false;
        }

        $metas = $this->get_option('woocommerceDataSyncMetas');
        if(empty($metas)) {
            return false;
        }

        $metas = explode(',', $metas);
        if(!in_array($meta_key, $metas)) {
            return;
        }
        
        $this->network_update_post_metas($object_id, $meta_key, $_meta_value);
    }

    public function network_update_post_metas($source_post_id, $meta_key, $meta_value)
    {
        $this->translation_tables->set_table('posts_translations');
        $translated_posts = $this->translation_tables->get_translated_items_by_item_id($this->current_blog_id, $source_post_id);
        if(empty($translated_posts)) {
            return false;
        }

        foreach ($translated_posts as $blog_id => $post_id) {
            if(empty($post_id) || $blog_id == $this->current_blog_id) {
                continue;
            }

            switch_to_blog($blog_id);

            update_post_meta($post_id, $meta_key, $meta_value);

            restore_current_blog();
        }
    }
}